require 'spec_helper'
include Sessions

describe BasicAuthSession do
  let(:session)  { Fabricate :basic_auth_session }
  let(:response) { double('response', code: 201, parsed_response: 'some_response') }

  describe '#initialize' do
    it 'sets the rom_host instance attribute' do
      session = Roqua::RomApi.basic_auth_session rom_host: 'some_rom_host'
      expect(session.rom_host).to eq('some_rom_host')
    end

    it 'defaults the rom_host to the ROM_SITE env variable' do
      orginal_env_rom_host = ENV['ROM_SITE']
      ENV['ROM_SITE'] = 'some_env_rom_host'
      session = Roqua::RomApi.basic_auth_session
      ENV['ROM_SITE'] = orginal_env_rom_host
      expect(session.rom_host).to eq('some_env_rom_host')
    end

    it 'sets the username instance variable' do
      session = Roqua::RomApi.basic_auth_session username: 'some_username'
      expect(session.username).to eq('some_username')
    end

    it 'defaults the username to the ROM_BASICAUTH_ID env variable' do
      orginal_env_rom_key = ENV['ROM_BASICAUTH_ID']
      ENV['ROM_BASICAUTH_ID'] = 'some_env_rom_key'
      session = Roqua::RomApi.basic_auth_session
      ENV['ROM_BASICAUTH_ID'] = orginal_env_rom_key
      expect(session.username).to eq('some_env_rom_key')
    end

    it 'sets the password instance variable' do
      session = Roqua::RomApi.basic_auth_session password: 'some_password'
      expect(session.password).to eq('some_password')
    end

    it 'defaults the password to the ROM_BASICAUTH_SECRET env variable' do
      orginal_env_rom_secret = ENV['ROM_BASICAUTH_SECRET']
      ENV['ROM_BASICAUTH_SECRET'] = 'some_env_rom_secret'
      session = Roqua::RomApi.basic_auth_session
      ENV['ROM_BASICAUTH_SECRET'] = orginal_env_rom_secret
      expect(session.password).to eq('some_env_rom_secret')
    end
  end

  describe '#get' do
    it 'performs a get request' do
      expect(HTTParty).to receive(:get).with('http://test.rom.roqua.eu/api/v1/some_path.json',
                                             query: {some: 'param'},
                                             basic_auth: {username: 'some_username', password: 'some_password'})
                                       .and_return(response)
      session.get '/some_path', some: 'param'
    end

    it 'throws an error if the reponse is not within the 200 range' do
      allow(response).to receive(:code).and_return(500)
      allow(HTTParty).to receive(:get).and_return(response)
      expect { session.get '/some_path' }.to raise_error(RuntimeError, 'some_response')
    end

    it 'throws an error if the reponse is not within the 200 range with json message' do
      response = double('response', code: 500, parsed_response: {'error' => 'the error'})
      allow(HTTParty).to receive(:get).and_return(response)
      expect { session.get '/some_path' }.to raise_error(RuntimeError, 'the error')
    end

    it 'throws a NoSession error if the basic auth is incorrect' do
      allow(response).to receive(:code).and_return(401)
      allow(HTTParty).to receive(:get).and_return(response)
      allow(response).to receive(:headers).and_return('WWW-Authenticate' => 'Basic realm="Application"')
      expect { session.get '/some_path' }.to raise_error(Roqua::RomApi::NoSession)
    end

    it 'throws a Unauthorized error on 401 without www-authenticate header' do
      allow(response).to receive(:code).and_return(401)
      allow(HTTParty).to receive(:get).and_return(response)
      allow(response).to receive(:headers).and_return('foo' => 'bar')
      expect { session.get '/some_path' }.to raise_error(Roqua::RomApi::Unauthorized)
    end

    it 'returns the response' do
      allow(HTTParty).to receive(:get).and_return(response)
      expect(session.get '/some_path').to eq(response)
    end
  end
end
